@@ -2,19 +2,20 @@
 /*  GMime
  *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -50,6 +51,7 @@ static ssize_t message_write_to_stream (GMimeObject *object, GMimeStream *stream
 
 static void message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string);
 
+static ssize_t write_structured (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_addrspec (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_received (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_subject (GMimeStream *stream, const char *name, const char *value);
@@ -144,7 +146,7 @@ g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass)
 	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Subject", write_subject);
 	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Received", write_received);
 	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Message-Id", write_msgid);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "References", write_addrspec);
+	g_mime_header_register_writer (((GMimeObject *) message)->headers, "References", write_structured);
 }
 
 static gboolean
@@ -547,7 +549,7 @@ write_msgid (GMimeStream *stream, const char *name, const char *value)
 }
 
 static ssize_t
-write_addrspec (GMimeStream *stream, const char *name, const char *value)
+write_structured (GMimeStream *stream, const char *name, const char *value)
 {
 	char *unfolded, *folded;
 	ssize_t n;
@@ -562,6 +564,29 @@ write_addrspec (GMimeStream *stream, const char *name, const char *value)
 	return n;
 }
 
+static ssize_t
+write_addrspec (GMimeStream *stream, const char *name, const char *value)
+{
+	InternetAddressList *addrlist;
+	GString *str;
+	ssize_t n;
+	
+	str = g_string_new (name);
+	g_string_append (str, ": ");
+	
+	if (value && (addrlist = internet_address_parse_string (value))) {
+		internet_address_list_writer (addrlist, str);
+		internet_address_list_destroy (addrlist);
+	}
+	
+	g_string_append_c (str, '\n');
+	
+	n = g_mime_stream_write (stream, str->str, str->len);
+	g_string_free (str, TRUE);
+	
+	return n;
+}
+
 
 enum {
 	HEADER_FROM,
@@ -1353,8 +1378,10 @@ g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
 	g_object_ref (mime_part);
 	g_mime_header_set_raw (mime_part->headers, NULL);
 	
-	if (message->mime_part)
+	if (message->mime_part) {
+		g_mime_header_set_raw (message->mime_part->headers, NULL);
 		g_object_unref (message->mime_part);
+	}
 	
 	message->mime_part = mime_part;
 }
