@@ -1,23 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
+/*  GMime
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -30,6 +28,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <errno.h>
 
 #include "gmime-multipart.h"
 #include "gmime-utils.h"
@@ -80,7 +79,7 @@ g_mime_multipart_get_type (void)
 			NULL, /* class_finalize */
 			NULL, /* class_data */
 			sizeof (GMimeMultipart),
-			16,   /* n_preallocs */
+			0,    /* n_preallocs */
 			(GInstanceInitFunc) g_mime_multipart_init,
 		};
 		
@@ -166,7 +165,7 @@ multipart_add_header (GMimeObject *object, const char *header, const char *value
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->add_header (object, header, value);
 }
 
@@ -174,13 +173,13 @@ static void
 multipart_set_header (GMimeObject *object, const char *header, const char *value)
 {
 	/* RFC 1864 states that you cannot set a Content-MD5 on a multipart */
-	if (!strcasecmp ("Content-MD5", header))
+	if (!g_ascii_strcasecmp ("Content-MD5", header))
 		return;
 	
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
 }
 
@@ -190,7 +189,7 @@ multipart_get_header (GMimeObject *object, const char *header)
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
 	else
 		return NULL;
@@ -202,7 +201,7 @@ multipart_remove_header (GMimeObject *object, const char *header)
 	/* Make sure that the header is a Content-* header, else it
            doesn't belong on a multipart */
 	
-	if (!strncasecmp ("Content-", header, 8))
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
 		GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
 }
 
@@ -238,8 +237,7 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 		g_mime_multipart_set_boundary (multipart, NULL);
 	
 	/* write the content headers */
-	nwritten = g_mime_header_write_to_stream (object->headers, stream);
-	if (nwritten == -1)
+	if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
 		return -1;
 	
 	total += nwritten;
@@ -252,8 +250,7 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 		
 		total++;
 		
-		nwritten = g_mime_stream_write_string (stream, multipart->preface);
-		if (nwritten == -1)
+		if ((nwritten = g_mime_stream_write_string (stream, multipart->preface)) == -1)
 			return -1;
 		
 		total += nwritten;
@@ -264,15 +261,13 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 		part = node->data;
 		
 		/* write the boundary */
-		nwritten = g_mime_stream_printf (stream, "\n--%s\n", multipart->boundary);
-		if (nwritten == -1)
+		if ((nwritten = g_mime_stream_printf (stream, "\n--%s\n", multipart->boundary)) == -1)
 			return -1;
 		
 		total += nwritten;
 		
 		/* write this part out */
-		nwritten = g_mime_object_write_to_stream (part, stream);
-		if (nwritten == -1)
+		if ((nwritten = g_mime_object_write_to_stream (part, stream)) == -1)
 			return -1;
 		
 		total += nwritten;
@@ -280,16 +275,14 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 		node = node->next;
 	}
 	
-	nwritten = g_mime_stream_printf (stream, "\n--%s--\n", multipart->boundary);
-	if (nwritten == -1)
+	if ((nwritten = g_mime_stream_printf (stream, "\n--%s--\n", multipart->boundary)) == -1)
 		return -1;
 	
 	total += nwritten;
 	
 	/* write the postface */
 	if (multipart->postface) {
-		nwritten = g_mime_stream_write_string (stream, multipart->postface);
-		if (nwritten == -1)
+		if ((nwritten = g_mime_stream_write_string (stream, multipart->postface)) == -1)
 			return -1;
 		
 		total += nwritten;
@@ -309,12 +302,12 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
  * multipart/mixed.
  **/
 GMimeMultipart *
-g_mime_multipart_new ()
+g_mime_multipart_new (void)
 {
 	GMimeMultipart *multipart;
 	GMimeContentType *type;
 	
-	multipart = g_object_new (GMIME_TYPE_MULTIPART, NULL, NULL);
+	multipart = g_object_new (GMIME_TYPE_MULTIPART, NULL);
 	
 	type = g_mime_content_type_new ("multipart", "mixed");
 	g_mime_object_set_content_type (GMIME_OBJECT (multipart), type);
@@ -339,7 +332,7 @@ g_mime_multipart_new_with_subtype (const char *subtype)
 	GMimeMultipart *multipart;
 	GMimeContentType *type;
 	
-	multipart = g_object_new (GMIME_TYPE_MULTIPART, NULL, NULL);
+	multipart = g_object_new (GMIME_TYPE_MULTIPART, NULL);
 	
 	type = g_mime_content_type_new ("multipart", subtype ? subtype : "mixed");
 	g_mime_object_set_content_type (GMIME_OBJECT (multipart), type);
@@ -635,18 +628,28 @@ g_mime_multipart_get_number (GMimeMultipart *multipart)
 
 
 static void
-read_random_pool (char *buffer, size_t bytes)
+read_random_pool (unsigned char *buffer, size_t bytes)
 {
+	size_t nread = 0;
+	ssize_t n;
 	int fd;
 	
-	fd = open ("/dev/urandom", O_RDONLY);
-	if (fd == -1) {
-		fd = open ("/dev/random", O_RDONLY);
-		if (fd == -1)
+	if ((fd = open ("/dev/urandom", O_RDONLY)) == -1) {
+		if ((fd = open ("/dev/random", O_RDONLY)) == -1)
 			return;
 	}
 	
-	read (fd, buffer, bytes);
+	do {
+		do {
+			n = read (fd, (char *) buffer + nread, bytes - nread);
+		} while (n == -1 && errno == EINTR);
+		
+		if (n == -1 || n == 0)
+			break;
+		
+		nread += n;
+	} while (nread < bytes);
+	
 	close (fd);
 }
 
@@ -657,14 +660,14 @@ multipart_set_boundary (GMimeMultipart *multipart, const char *boundary)
 	
 	if (!boundary) {
 		/* Generate a fairly random boundary string. */
-		char digest[16], *p;
-		int state, save;
+		unsigned char digest[16], *p;
+		guint32 save = 0;
+		int state = 0;
 		
 		read_random_pool (digest, 16);
 		
 		strcpy (bbuf, "=-");
-		p = bbuf + 2;
-		state = save = 0;
+		p = (unsigned char *) bbuf + 2;
 		p += g_mime_utils_base64_encode_step (digest, 16, p, &state, &save);
 		*p = '\0';
 		
